use std::io::Cursor;
use std::path::PathBuf;

use crate::sftp;
use crate::modpack;
use ssh2::Sftp;
use ssh2::Session;

//static USERNAME: parking_lot::Mutex<String> = parking_lot::const_mutex(String::new());
//static PASSWORD: parking_lot::Mutex<String> = parking_lot::const_mutex(String::new());
static SESSION: tauri::async_runtime::Mutex<Option<Session>> = tauri::async_runtime::Mutex::const_new(None);

#[tauri::command]
pub async fn login(username: String, password: String, window: tauri::Window) {
    let res = sftp::connect(username, password);
    if(res.is_ok()){
        //*USERNAME.lock() = username;
        //*PASSWORD.lock() = password;
        *SESSION.lock().await = Some(res.unwrap());
        window.emit("Login_Success", {});
    }else{
        window.emit("Login_Failed", {});
    }
}

#[tauri::command]
pub async fn drop_session(){
    let ref mut session = *SESSION.lock().await;
    if let Some(session) = session {
        session.disconnect(None, "disconnecting", None);
    }
    *session = None;
}

async fn update_modpacks(modpacks: Vec<modpack::ModpackEntry>) -> Result<(), String>{
    let data = serde_json::to_string(&modpacks).or(Err("Unable to serialize json"))?;
    let reader = Cursor::new(data.as_bytes());
    let ref mut session = *SESSION.lock().await;
    if let Some(session) = session {
        sftp::uplaod(None, session.clone(), PathBuf::from("/ftp/modpacks.json"), reader, format!("modpacks.json"), data.as_bytes().len()).await;
    }
    Ok(())
}

#[tauri::command]
pub async fn shift_up(id: String){
    let mut modpacks = modpack::get_modpacks().await;
    let mut index = 0;
    for pack in modpacks.as_slice() {
        if(pack.id == id){
            break;
        }
        index += 1;
    }
    if index != 0 {
        modpacks.swap(index, index-1);
    }
    update_modpacks(modpacks).await;
}

#[tauri::command]
pub async fn shift_down(id: String){
    let mut modpacks = modpack::get_modpacks().await;
    let mut index = 0;
    for pack in modpacks.as_slice() {
        if(pack.id == id){
            break;
        }
        index += 1;
    }
    if index != modpacks.len()-1 {
        modpacks.swap(index, index+1);
    }
    update_modpacks(modpacks).await;
}